package javax.swing.tree;

import java.beans.ConstructorProperties;
import java.io.Serializable;

public class TreePath implements Serializable
{
    private TreePath parentPath;
    private Object lastPathComponent;
    
    @ConstructorProperties({ "path" })
    public TreePath(Object[] array) {
        if (array == null || array.length == 0) {
            throw new IllegalArgumentException("path in TreePath must be non null and not empty.");
        }
        this.lastPathComponent = array[array.length - 1];
        if (this.lastPathComponent == null) {
            throw new IllegalArgumentException("Last path component must be non-null");
        }
        if (array.length > 1) {
            this.parentPath = new TreePath(array, array.length - 1);
        }
        return;
    }
    
    public TreePath(Object lastPathComponent) {
        if (lastPathComponent == null) {
            throw new IllegalArgumentException("path in TreePath must be non null.");
        }
        this.lastPathComponent = lastPathComponent;
        this.parentPath = null;
        return;
    }
    
    protected TreePath(TreePath parentPath, Object lastPathComponent) {
        if (lastPathComponent == null) {
            throw new IllegalArgumentException("path in TreePath must be non null.");
        }
        this.parentPath = parentPath;
        this.lastPathComponent = lastPathComponent;
        return;
    }
    
    protected TreePath(Object[] array, int n) {
        this.lastPathComponent = array[n - 1];
        if (this.lastPathComponent == null) {
            throw new IllegalArgumentException("Path elements must be non-null");
        }
        if (n > 1) {
            this.parentPath = new TreePath(array, n - 1);
        }
        return;
    }
    
    protected TreePath() {
    }
    
    public Object[] getPath() {
        int pathCount = this.getPathCount();
        Object[] array = new Object[pathCount--];
        while (this != null) {
            array[pathCount--] = this.getLastPathComponent();
            this = this.getParentPath();
        }
        return array;
    }
    
    public Object getLastPathComponent() {
        return this.lastPathComponent;
    }
    
    public int getPathCount() {
        int n = 0;
        while (this != null) {
            ++n;
            this = this.getParentPath();
        }
        return n;
    }
    
    public Object getPathComponent(int i) {
        int pathCount = this.getPathCount();
        if (i < 0 || i >= pathCount) {
            throw new IllegalArgumentException("Index " + i + " is out of the specified range");
        }
        for (int j = pathCount - 1; j != i; --j) {
            this = this.getParentPath();
        }
        return this.getLastPathComponent();
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if ((o instanceof TreePath)) {
            TreePath parentPath = (TreePath)o;
            if (this.getPathCount() != parentPath.getPathCount()) {
                return false;
            }
            while (this != null) {
                if (!this.getLastPathComponent().equals(parentPath.getLastPathComponent())) {
                    return false;
                }
                parentPath = parentPath.getParentPath();
                this = this.getParentPath();
            }
            return true;
        }
        else {
            return false;
        }
    }
    
    @Override
    public int hashCode() {
        return this.getLastPathComponent().hashCode();
    }
    
    public boolean isDescendant(TreePath parentPath) {
        if (parentPath == this) {
            return true;
        }
        if (parentPath != null) {
            int pathCount = this.getPathCount();
            int pathCount2 = parentPath.getPathCount();
            if (pathCount2 < pathCount) {
                return false;
            }
            while (pathCount2-- > pathCount) {
                parentPath = parentPath.getParentPath();
            }
            return this.equals(parentPath);
        }
        else {
            return false;
        }
    }
    
    public TreePath pathByAddingChild(Object o) {
        if (o == null) {
            throw new NullPointerException("Null child not allowed");
        }
        return new TreePath(this, o);
    }
    
    public TreePath getParentPath() {
        return this.parentPath;
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer("[");
        for (int i = 0; i < this.getPathCount(); ++i) {
            if (i > 0) {
                sb.append(", ");
            }
            sb.append(this.getPathComponent(i));
        }
        sb.append("]");
        return sb.toString();
    }
}
